module TimeInputs
  class Parser
    def initialize error_policy=->(message){ raise BaseError, message }
      @error_policy = error_policy
    end

    def parse input_string, options={}
      input_as_hh_mm = to_hh_mm(input_string.to_s)
      date_context = check_date! options.fetch(:date_context) { Date.today }
      hours, minutes = input_as_hh_mm.split(":")
      Time.local(date_context.year, date_context.month, date_context.day, hours.to_i, minutes.to_i)
    rescue BaseError => e
      @error_policy.call(e.message)
    end

    private
    def to_hh_mm input
      raise BaseError, "invalid input" unless input =~ /\A\d{1,2}:?\d{2}\z/
      input.rjust(4,'0').tap do |str|
        str.insert(2,':') unless str =~/\A\d?\d:\d{2}\z/
      end
    end

    def check_date! date
      raise BaseError, "invalid date_context" unless date.respond_to?(:to_date)
      date.to_date
    end
  end
end
